#ifndef NO_GSSAPI_CONFIG_H
#include "gssapi_config.h"
#else
#define GLOBUS_RELEASE_MAJOR 0
#define GLOBUS_RELEASE_MINOR 0
#define GLOBUS_RELEASE_BETA  0
#define GLOBUS_RELEASE_PATCH 0
#endif

#define DEFAULT_SECURE_TMP_DIR "/tmp"
#define FILE_SEPERATOR "/"

#include <sslutils.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <buffer.h>
#include <crypto.h>
#include <objects.h>
#include <asn1.h>
#include <evp.h>
#include <x509.h>
#include <pem.h>
#include <ssl.h>
#include <rsa.h>

#ifdef USE_PKCS11
#include <scutils.h>
#endif

static void kpcallback(int p, int n)
{
    char c='B';

    if (p == 0) c='.';
    if (p == 1) c='+';
    if (p == 2) c='*';
    if (p == 3) c='\n';
    fputc(c,stderr);
}

int main(int argc, char *argv[]) {

  int                  bits        = 512;   /* default proxy to 512 bits */
  int                  hours       = 12;    /* default to a 12 hour cert */
  int                  limit_proxy = 0;     /* dont restrict the proxy */
  int                  debug       = 0;
  int                  verify      = 1;
  int                  i;
  int		       j;
  char *               certfile    = /* argv[3] */ NULL;
  char *               keyfile     = /* argv[4] */ NULL;
  char *               outfile     = /* argv[5] */ NULL;
  char *               certdir     = /* argv[1] */ NULL;
  char *               certcafile  = /* argv[2] */ NULL;
  char *               pin         = NULL;

  proxy_cred_desc *    pcd         = NULL;
  X509_STORE *         cert_ctx    = NULL;
  X509_LOOKUP *        lookup      = NULL;
  X509_STORE_CTX       csc;
  BIO *                bio_err;
  X509 *               xcert;

  char *x509_cert_dir; 
  char *x509_cert_file;
  char *x509_user_cert;
  char *x509_user_key;
  char *x509_user_proxy;
  char *tkt_ccache; 

  int (*pw_cb)() = NULL;

  if(argc != 7) {
    fprintf(stderr, "wrong number of arguments: %d\n", argc);
    exit(-1);
  }

  x509_cert_dir = (char *)malloc(strlen("X509_CERT_DIR")+strlen(argv[1])+2);
  sprintf(x509_cert_dir, "%s=%s", "X509_CERT_DIR", argv[1]);
  fprintf(stderr, "%s\n", x509_cert_dir);
 
  x509_cert_file = (char *)malloc(strlen("X509_CERT_FILE")+strlen(argv[2])+2);
  sprintf(x509_cert_file, "%s=%s", "X509_CERT_FILE", argv[2]);
  fprintf(stderr, "%s\n", x509_cert_file);

  x509_user_cert = (char *)malloc(strlen("X509_USER_CERT")+strlen(argv[3])+2);
  sprintf(x509_user_cert, "%s %s", "X509_USER_CERT", argv[3]);
  fprintf(stderr, "%s\n", x509_user_cert);

  x509_user_key = (char *)malloc(strlen("X509_USER_KEY")+strlen(argv[4])+2);
  sprintf(x509_user_key, "%s %s", "X509_USER_KEY", argv[4]);
  fprintf(stderr, "%s\n", x509_user_key);

  x509_user_proxy = (char *)malloc(strlen("X509_USER_PROXY")+strlen(argv[5])+2);
  sprintf(x509_user_proxy, "%s=%s", "X509_USER_PROXY", argv[5]);
  fprintf(stderr, "%s\n", x509_user_proxy);

  tkt_ccache = (char *)malloc(strlen("KRB5CCNAME")+strlen(argv[6])+2);
  sprintf(tkt_ccache, "%s=%s", "KRB5CCNAME", argv[6]);
  fprintf(stderr, "%s\n", tkt_ccache);

  fprintf(stderr, "setting enviromental variables\n");

  if(putenv(x509_cert_dir) != 0) 
	fprintf(stderr, "unable to set %s\n", x509_cert_dir);
  if(putenv(x509_cert_file) != 0)
	fprintf(stderr, "unable to set %s\n", x509_cert_file);
  if(putenv(x509_user_cert) != 0 )
	fprintf(stderr, "unable to set %s\n", x509_user_cert);
  if(putenv(x509_user_key) != 0)
	fprintf(stderr, "unable to set %s\n", x509_user_key);
  if(putenv(x509_user_proxy) != 0)
	fprintf(stderr, "unable to set %s\n", x509_user_proxy);
  if(putenv(tkt_ccache) != 0)
	fprintf(stderr, "unable to set %s\n", tkt_ccache);

  ERR_load_prxyerr_strings(0);
  SSLeay_add_ssl_algorithms();

  if ((bio_err=BIO_new(BIO_s_file())) != NULL)
    BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);

  if ((pcd = proxy_cred_desc_new()) == NULL)
    goto err;

  if ( proxy_get_filenames(0, &certcafile, &certdir, &outfile, &certfile,
			   &keyfile) )
    goto err;

  if (certdir) pcd->certdir = strdup(certdir);

  if (proxy_load_user_cert(pcd, certfile, pw_cb)) goto err;

  if (proxy_load_user_key(pcd, keyfile, pw_cb)) goto err;

  if (strncmp("SC",certfile,2) && !strcmp(certfile, keyfile))  {
    if (pcd->cert_chain == NULL)
      pcd->cert_chain = sk_new_null();
    if (proxy_load_user_proxy(pcd->cert_chain, certfile) < 0)
      goto err;
  } 

  if (proxy_create_local(pcd, outfile, hours, bits, limit_proxy,
			 (int (*)(void)) kpcallback))
    goto err;

  return 0;

err:
  {
    unsigned long l;
    char buf[256];
    char *file;
    const char *data;
    int line;
    
    /* WIN32 does not have the ERR_get_error_line_data */ 
    /* exported, so simulate it till it is fixed */
    /* in SSLeay-0.9.0 */
    
    while ( ERR_peek_error() != 0 ) {
      int i;
      ERR_STATE *es;
      
      es = ERR_get_state();
      i=(es->bottom+1)%ERR_NUM_ERRORS;
      
      if (es->err_data[i] == NULL)
	data = "";
      else
	data = es->err_data[i];
      
      l=ERR_get_error_line(&file,&line);
      if (debug)
	fprintf(stderr,
		"%s:%s:%d",
		ERR_error_string(l,buf),
		file,
		line);
      else
	fprintf(stderr,
		"%s:%s",
		ERR_func_error_string(l),
		ERR_reason_error_string(l));
      
      fprintf(stderr,"%s",data);
      fprintf(stderr,"\n");
    }
  }
  exit(1);
}
